/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "Application.hpp"

// Set the application to null for the linker
Application* Application::TheApplication = 0;

/**
 * The constructor for the Application class
 *
 * \param windowTitle_ The title of the window
 * \param xPosition_ The initial x position of the window
 * \param yPosition_ The initial y position of the window
 * \param width_ The width of the window
 * \param height_ The height of the window
 * \param backgroundColor_ The background color of the window
 */
Application::Application(const char* windowTitle_, int xPosition_, int yPosition_, int width_, int height_)
: windowTitle(windowTitle_)
{
	xPosition = xPosition_;
	yPosition = yPosition_;

	width  = width_;
	height = height_;
}

/**
 * Destructor for the Application class
 */
Application::~Application()
{

}

/**
 * The entrypoint into the application.
 *
 * \argumentCount The number of arguments passed
 * \argumentValues The values of the arguments passed
 */
int Application::Run(int argumentCount, char** argumentValues)
{
	Application* app = (Application*)Application::TheApplication;

	return(app->main(argumentCount, argumentValues));
}

/**
 * Defines what actions to take on entry into the application.
 *
 * \note This is meant to contain all the initialization of the application.
 */
void Application::onEntry()
{

}

/**
 * Define what actions to take when exiting the application.
 *
 * \note This is meant to contain all the cleanup of the application.
 */
void Application::onClose()
{

}

/**
 * Display loop of the application.
 */
void Application::onDisplay()
{

}

/**
 * Idle loop of the application.
 *
 * \note The actual logic of the program is meant to be here.
 */
void Application::onIdle()
{

}

/**
 * Reshape callback of the application.
 *
 * \param width_ The new width of the window
 * \param height_ The new height of the window
 */
void Application::onReshape(int width_, int height_)
{
	width = width_;
	height = height_;
}

/**
 * Key released callback of the application
 *
 * \param key The key released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onKeyUp(unsigned char key, int x, int y)
{

}

/**
 * Key pressed callback of the application
 *
 * \param key The key pressed
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onKeyDown(unsigned char key, int x, int y)
{

}

/**
 * Special key released callback of the application
 *
 * \param key The key released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onSpecialKeyUp(int key, int x, int y)
{

}

/**
 * Special key pressed callback of the application
 *
 * \param key The key pressed
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onSpecialKeyDown(int key, int x, int y)
{

}

/**
 * Mouse click callback of the application
 *
 * \param button The mouse button pressed
 * \param upOrDown Whether the button was pressed or released
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onMouseClick(int button, int upOrDown, int x, int y)
{

}

/**
 * Mouse wheel moved callback of the application
 *
 * \param wheel The mouse wheel moved
 * \param direction The direction it was moved
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onMouseWheel(int wheel, int direction, int x, int y)
{

}

/**
 * Mouse pressed and moved callback of the application
 *
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onMotion(int x, int y)
{

}

/**
 * Mouse moved callback of the application
 *
 * \param x The x coordinate of the mouse relative to the window
 * \param x The y coordinate of the mouse relative to the window
 */
void Application::onPassiveMotion(int x, int y)
{

}
